@@ -6,17 +6,23 @@ import apt_pkg
 import os
 import re
 import logging
+from subprocess import Popen, PIPE
+
 from gettext import gettext as _
 from DistUpgradeConfigParser import DistUpgradeConfig
 from DistUpgradeView import FuzzyTimeToStr
 
+# FIXME: we need this only for the later "isinstance()" check
+#        this should probably be solved in some different way
+from DistUpgradeViewText import DistUpgradeViewText
+
 class MyCache(apt.Cache):
     # init
-    def __init__(self, config, progress=None):
+    def __init__(self, config, view, progress=None):
         apt.Cache.__init__(self, progress)
         self.to_install = []
         self.to_remove = []
-
+        self.view = view
         self.config = config
         self.metapkgs = self.config.getlist("Distro","MetaPkgs")
 
@@ -163,11 +169,24 @@ class MyCache(apt.Cache):
         if func is not None:
             func()
 
+    def feistyQuirks(self):
+        """ this function works around quirks in the edgy->feisty upgrade """
+        logging.debug("running feistyQuirks handler")
+        # ndisrwapper changed again *sigh*
+        for (fr, to) in [("ndiswrapper-utils-1.8","ndiswrapper-utils-1.9")]:
+            if self.has_key(fr) and self.has_key(to):
+                if self[fr].isInstalled and not self[to].markedInstall:
+                    try:
+                        self.markInstall(to,"%s->%s quirk upgrade rule" % (fr, to))
+                    except SystemError, e:
+                        logging.warning("Failed to apply %s->%s install (%s)" % (fr, to, e))
+            
+
     def edgyQuirks(self):
         """ this function works around quirks in the dapper->edgy upgrade """
         logging.debug("running edgyQuirks handler")
-        # deal with the python2.4-$foo -> python-$foo transition
         for pkg in self:
+            # deal with the python2.4-$foo -> python-$foo transition
             if (pkg.name.startswith("python2.4-") and
                 pkg.isInstalled and
                 not pkg.markedUpgrade):
@@ -178,15 +197,51 @@ class MyCache(apt.Cache):
                                          "python2.4->python upgrade rule")
                     except SystemError, e:
                         logging.debug("Failed to apply python2.4->python install: %s (%s)" % (basepkg, e))
-        # deal with *gar*gar* hpijs
-        if (self.has_key("hpijs") and self["hpijs"].isInstalled and
-            not self["hpijs"].markedUpgrade):
-            try:
-                self.markInstall("hpijs","hpijs quirk upgrade rule")
-            except SystemError, e:
-                logging.debug("Failed to apply hpijs install (%s)" % e)
+            # xserver-xorg-input-$foo gives us trouble during the upgrade too
+            if (pkg.name.startswith("xserver-xorg-input-") and
+                pkg.isInstalled and
+                not pkg.markedUpgrade):
+                try:
+                    self.markInstall(pkg.name, "xserver-xorg-input fixup rule")
+                except SystemError, e:
+                    logging.debug("Failed to apply fixup: %s (%s)" % (pkg.name, e))
             
-        
+        # deal with held-backs that are unneeded
+        for pkgname in ["hpijs", "bzr", "tomboy"]:
+            if (self.has_key(pkgname) and self[pkgname].isInstalled and
+                not self[pkgname].markedUpgrade):
+                try:
+                    self.markInstall(pkgname,"%s quirk upgrade rule" % pkgname)
+                except SystemError, e:
+                    logging.debug("Failed to apply %s install (%s)" % (pkgname,e))
+        # libgl1-mesa-dri from xgl.compiz.info (and friends) breaks the
+	# upgrade, work around this here by downgrading the package
+        if self.has_key("libgl1-mesa-dri"):
+            pkg = self["libgl1-mesa-dri"]
+            # the version from the compiz repo has a "6.5.1+cvs20060824" ver
+            if (pkg.candidateVersion == pkg.installedVersion and
+                "+cvs2006" in pkg.candidateVersion):
+                for ver in pkg._pkg.VersionList:
+                    # the "officual" edgy version has "6.5.1~20060817-0ubuntu3"
+                    if "~2006" in ver.VerStr:
+			# ensure that it is from a trusted repo
+			for (VerFileIter, index) in ver.FileList:
+				indexfile = self._list.FindIndex(VerFileIter)
+				if indexfile and indexfile.IsTrusted:
+					logging.info("Forcing downgrade of libgl1-mesa-dri for xgl.compz.info installs")
+		                        self._depcache.SetCandidateVer(pkg._pkg, ver)
+					break
+                                    
+        # deal with general if $foo is installed, install $bar
+        for (fr, to) in [("xserver-xorg-driver-all","xserver-xorg-video-all")]:
+            if self.has_key(fr) and self.has_key(to):
+                if self[fr].isInstalled and not self[to].markedInstall:
+                    try:
+                        self.markInstall(to,"%s->%s quirk upgrade rule" % (fr, to))
+                    except SystemError, e:
+                        logging.debug("Failed to apply %s->%s install (%s)" % (fr, to, e))
+                    
+                    
                                   
     def dapperQuirks(self):
         """ this function works around quirks in the breezy->dapper upgrade """
@@ -197,7 +252,36 @@ class MyCache(apt.Cache):
             self.markRemove("nvidia-settings")
             self.markInstall("nvidia-glx")
 
-    def distUpgrade(self, view):
+    def checkForKernel(self):
+        """ check for the runing kernel and try to ensure that we have
+            a updated version
+        """
+        uname = Popen(["uname","-r"],stdout=PIPE).communicate()[0].strip()
+        logging.debug("Kernel uname: '%s' " % uname)
+        try:
+            (version, build, flavour) = uname.split("-")
+        except Exception, e:
+            logging.warning("Can't parse kernel uname: '%s' (self compiled?)" % e)
+            return False
+        kernel = "linux-image-%s" % flavour
+        if not self.has_key(kernel):
+            logging.warning("No kernel: '%s'" % kernel)
+            return False
+        if not (self[kernel].isInstalled or self[kernel].markedInstall):
+            logging.debug("Selecting new kernel '%s'" % kernel)
+            self[kernel].markInstall()
+        return True
+
+    def distUpgrade(self, view, serverMode, logfd):
+        def _restore_fds(stdout, stderr):
+            os.dup2(stdout, 1)
+            os.dup2(stderr, 2)
+        text_mode = isinstance(view, DistUpgradeViewText)
+        if text_mode:
+            old_stdout = os.dup(1)
+            old_stderr = os.dup(2)
+            os.dup2(logfd, 1)
+            os.dup2(logfd, 2)
         try:
             # upgrade (and make sure this way that the cache is ok)
             self.upgrade(True)
@@ -208,12 +292,17 @@ class MyCache(apt.Cache):
             # and if we have some special rules
             self.postUpgradeRule()
 
-            # then see if meta-pkgs are missing
-            if not self._installMetaPkgs(view):
-                raise SystemError, _("Can't upgrade required meta-packages")
+            # check if we got a new kernel
+            self.checkForKernel()
+
+            # install missing meta-packages (if not in server upgrade mode)
+            if not serverMode:
+                if not self._installMetaPkgs(view):
+                    raise SystemError, _("Can't upgrade required meta-packages")
 
             # see if it all makes sense
             if not self._verifyChanges():
+                if text_mode: _restore_fds(old_stdout, old_stderr)
                 raise SystemError, _("A essential package would have to be removed")
         except SystemError, e:
             # FIXME: change the text to something more useful
@@ -224,6 +313,7 @@ class MyCache(apt.Cache):
                          "package and include the files in /var/log/dist-upgrade/ "
                          "in the bugreport."))
             logging.error("Dist-upgrade failed: '%s'", e)
+            if text_mode: _restore_fds(old_stdout, old_stderr)
             return False
 
         # check the trust of the packages that are going to change
@@ -231,6 +321,15 @@ class MyCache(apt.Cache):
         for pkg in self.getChanges():
             if pkg.markedDelete:
                 continue
+            # special case because of a bug in pkg.candidateOrigin
+            if pkg.markedDowngrade:
+                for ver in pkg._pkg.VersionList:
+                    # version is lower than installed one
+                    if apt_pkg.VersionCompare(ver.VerStr, pkg.installedVersion) < 0:
+                        for (verFileIter,index) in ver.FileList:
+                            if not origin.trusted:
+                                untrusted.append(pkg.name)
+                continue
             origins = pkg.candidateOrigin
             trusted = False
             for origin in origins:
@@ -249,7 +348,9 @@ class MyCache(apt.Cache):
                          "You may want to try again later. See below for a "
                          "list of unauthenticated packages."),
                        "\n".join(untrusted))
+            if text_mode: _restore_fds(old_stdout, old_stderr)
             return False
+        if text_mode: _restore_fds(old_stdout, old_stderr)
         return True
 
     def _verifyChanges(self):
@@ -267,6 +368,42 @@ class MyCache(apt.Cache):
                 return False
         return True
 
+    @property
+    def installedTasks(self):
+        tasks = {}
+        installed_tasks = set()
+        for pkg in self:
+            pkg._lookupRecord()
+            for line in pkg._records.Record.split("\n"):
+                if line.startswith("Task:"):
+                    for task in (line[len("Task:"):]).split(","):
+                        task = task.strip()
+                        if not tasks.has_key(task):
+                            tasks[task] = set()
+                        tasks[task].add(pkg.name)
+        for task in tasks:
+            installed = True
+            for pkgname in tasks[task]:
+                if not self[pkgname].isInstalled:
+                    installed = False
+                    break
+            if installed:
+                installed_tasks.add(task)
+        return installed_tasks
+            
+    def installTasks(self, tasks):
+        for pkg in self:
+            if pkg.markedInstall or pkg.isInstalled:
+                continue
+            pkg._lookupRecord()
+            for line in pkg._records.Record.split("\n"):
+                if line.startswith("Task:"):
+                    for task in (line[len("Task:"):]).split(","):
+                        task = task.strip()
+                        if task in tasks:
+                            pkg.markInstall()
+        return True
+    
     def _installMetaPkgs(self, view):
         # helper for this func
         def metaPkgInstalled():
@@ -298,7 +435,7 @@ class MyCache(apt.Cache):
                 return False
         # check if we have a meta-pkg, if not, try to guess which one to pick
         if not metaPkgInstalled():
-            logging.debug("no {ubuntu,edubuntu,kubuntu}-desktop pkg installed")
+            logging.debug("none of the '%s' meta-pkgs installed" % metapkgs)
             for key in metapkgs:
                 deps_found = True
                 for pkg in self.config.getlist(key,"KeyDependencies"):
@@ -319,10 +456,10 @@ class MyCache(apt.Cache):
             # FIXME: provide a list
             view.error(_("Can't guess meta-package"),
                        _("Your system does not contain a "
-                         "ubuntu-desktop, kubuntu-desktop or "
+                         "ubuntu-desktop, kubuntu-desktop, xubuntu-desktop or "
                          "edubuntu-desktop package and it was not "
                          "possible to detect which version of "
-                        "ubuntu you are running.\n "
+                        "Ubuntu you are running.\n "
                          "Please install one of the packages "
                          "above first using synaptic or "
                          "apt-get before proceeding."))
@@ -336,6 +473,10 @@ class MyCache(apt.Cache):
         return False
 
     def _tryMarkObsoleteForRemoval(self, pkgname, remove_candidates, foreign_pkgs):
+        # sanity check
+        if self._inRemovalBlacklist(pkgname):
+            logging.debug("skipping '%s' (in removalBlacklist)" % pkgname)
+            return False
         # this is a delete candidate, only actually delete,
         # if it dosn't remove other packages depending on it
         # that are not obsolete as well
@@ -343,13 +484,13 @@ class MyCache(apt.Cache):
         try:
             self[pkgname].markDelete()
             for pkg in self.getChanges():
-                if pkg.name not in remove_candidates or \
-                       pkg.name in foreign_pkgs or \
-                       self._inRemovalBlacklist(pkg.name):
+                if (pkg.name not in remove_candidates or 
+                      pkg.name in foreign_pkgs or 
+                      self._inRemovalBlacklist(pkg.name)):
                     self.restore_snapshot()
                     return False
         except (SystemError,KeyError),e:
-            loggging.warning("_tryMarkObsoleteForRemoval failed for '%s' (%s)" % (pkgname,e))
+            logging.warning("_tryMarkObsoleteForRemoval failed for '%s' (%s: %s)" % (pkgname, repr(e), e))
             self.restore_snapshot()
             return False
         return True
@@ -388,3 +529,6 @@ if __name__ == "__main__":
 	import DistUpgradeConfigParser
 	c = MyCache(DistUpgradeConfigParser.DistUpgradeConfig("."))
 	c.clear()
+        c.installedTasks
+        c.installTasks(["ubuntu-desktop"])
+        print c.getChanges()
