@@ -36,8 +36,9 @@ import apt
 import apt_pkg
 import os
 
-from apt.progress import InstallProgress
-from DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, estimatedDownloadTime
+from DistUpgradeApport import *
+
+from DistUpgradeView import DistUpgradeView, FuzzyTimeToStr, estimatedDownloadTime, InstallProgress
 from UpdateManager.Common.SimpleGladeApp import SimpleGladeApp, bindtextdomain
 
 import gettext
@@ -126,7 +127,7 @@ class GtkFetchProgressAdapter(apt.progress.FetchProgress):
             currentItem = self.totalItems
 
         if self.currentCPS > 0:
-            self.status.set_text(_("Fetching file %li of %li at %s/s") % (currentItem, self.totalItems, apt_pkg.SizeToStr(self.currentCPS)))
+            self.status.set_text(_("Fetching file %li of %li at %sb/s") % (currentItem, self.totalItems, apt_pkg.SizeToStr(self.currentCPS)))
             self.progress.set_text(_("About %s remaining") % FuzzyTimeToStr(self.eta))
         else:
             self.status.set_text(_("Fetching file %li of %li") % (currentItem, self.totalItems))
@@ -154,8 +155,10 @@ class GtkInstallProgressAdapter(InstallProgress):
         reaper.connect("child-exited", self.child_exited)
         # some options for dpkg to make it die less easily
         apt_pkg.Config.Set("DPkg::Options::","--force-overwrite")
+        apt_pkg.Config.Set("DPkg::StopOnError","False")
 
     def startUpdate(self):
+        InstallProgress.startUpdate(self)
         self.finished = False
         # FIXME: add support for the timeout
         # of the terminal (to display something useful then)
@@ -181,6 +184,7 @@ class GtkInstallProgressAdapter(InstallProgress):
         self.last_activity = 0.0
         
     def error(self, pkg, errormsg):
+        InstallProgress.error(self, pkg, errormsg)
         logging.error("got an error from dpkg for pkg: '%s': '%s'" % (pkg, errormsg))
         #self.expander_terminal.set_expanded(True)
         self.parent.dialog_error.set_transient_for(self.parent.window_main)
@@ -237,7 +241,7 @@ class GtkInstallProgressAdapter(InstallProgress):
         if self.start_time == 0.0:
           #print "setting start time to %s" % self.start_time
           self.start_time = time.time()
-        self.progress.set_fraction(float(self.percent)/100.0)
+        self.progress.set_fraction(float(percent)/100.0)
         self.label_status.set_text(status.strip())
         # start showing when we gathered some data
         if percent > 1.0:
@@ -247,7 +251,7 @@ class GtkInstallProgressAdapter(InstallProgress):
           # time wasted in conffile questions (or other ui activity)
           delta -= self.time_ui
           time_per_percent = (float(delta)/percent)
-          eta = (100.0 - self.percent) * time_per_percent
+          eta = (100.0 - percent) * time_per_percent
           # only show if we have some sensible data (60sec < eta < 2days)
           if eta > 61.0 and eta < (60*60*24*2):
             self.progress.set_text(_("About %s remaining") % FuzzyTimeToStr(eta))
@@ -267,7 +271,12 @@ class GtkInstallProgressAdapter(InstallProgress):
         self.label_status.set_text("")
     
     def updateInterface(self):
-        InstallProgress.updateInterface(self)
+        try:
+          InstallProgress.updateInterface(self)
+        except ValueError, e:
+          logging.error("got ValueError from InstallPrgoress.updateInterface. Line was '%s' (%s)" % (self.read, e))
+          # reset self.read so that it can continue reading and does not loop
+	  self.read = ""
         # check if we haven't started yet with packages, pulse then
         if self.start_time == 0.0:
           self.progress.pulse()
@@ -287,13 +296,15 @@ class DistUpgradeVteTerminal(object):
   def __init__(self, parent, term):
     self.term = term
     self.parent = parent
-  def call(self, cmd):
+  def call(self, cmd, hidden=False):
     def wait_for_child(widget):
       #print "wait for child finished"
       self.finished=True
     self.term.show()
     self.term.connect("child-exited", wait_for_child)
-    self.parent.expander_terminal.set_expanded(True)
+    self.parent.expander_terminal.set_sensitive(True)
+    if hidden==False:
+      self.parent.expander_terminal.set_expanded(True)
     self.term.fork_command(command=cmd[0],argv=cmd)
     self.finished = False
     while not self.finished:
@@ -320,7 +331,11 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
           logging.warning("Error setting locales (%s)" % e)
         
         icons = gtk.icon_theme_get_default()
-        gtk.window_set_default_icon(icons.load_icon("update-manager", 32, 0))
+        try:
+          gtk.window_set_default_icon(icons.load_icon("update-manager", 32, 0))
+        except gobject.GError, e:
+          logging.debug("error setting default icon, ignoring (%s)" % e)
+          pass
         SimpleGladeApp.__init__(self, gladedir+"/DistUpgrade.glade",
                                 None, domain="update-manager")
         self.prev_step = 0 # keep a record of the latest step
@@ -331,6 +346,11 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         # -> this avoid the issue that during the dapper->edgy upgrade
         #    the loaders move from /usr/lib/gtk/2.4.0/loaders to 2.10.0
         self.pngloader = gtk.gdk.PixbufLoader("png")
+        try:
+          self.svgloader = gtk.gdk.PixbufLoader("svg")
+        except gobject.GError, e:
+          logging.warning("svg pixbuf loader failed (%s)" % e)
+          pass
         
         self.window_main.realize()
         self.window_main.window.set_functions(gtk.gdk.FUNC_MOVE)
@@ -359,17 +379,25 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         sys.excepthook = self._handleException
 
     def _handleException(self, type, value, tb):
+      # we handle the exception here, hand it to apport and run the
+      # apport gui manually after it because we kill u-m during the upgrade
+      # to prevent it from poping up for reboot notifications or FF restart
+      # notifications or somesuch
       import traceback
       lines = traceback.format_exception(type, value, tb)
       logging.error("not handled expection:\n%s" % "\n".join(lines))
-      self.error(_("A fatal error occured"),
-                 _("Please report this as a bug and include the "
-                   "files /var/log/dist-upgrade/main.log and "
-                   "/var/log/dist-upgrade/apt.log "
-                   "in your report. The upgrade aborts now.\n"
-                   "Your original sources.list was saved in "
-                   "/etc/apt/sources.list.distUpgrade."),
-                 "\n".join(lines))
+      # we can't be sure that apport will run in the middle of a upgrade
+      # so we still show a error message here
+      apport_crash(type, value, tb)
+      if not run_apport():
+        self.error(_("A fatal error occured"),
+                   _("Please report this as a bug (if you haven't already) and include the"
+                     "files /var/log/dist-upgrade/main.log and "
+                     "/var/log/dist-upgrade/apt.log "
+                     "in your report. The upgrade aborts now.\n"
+                     "Your original sources.list was saved in "
+                     "/etc/apt/sources.list.distUpgrade."),
+                   "\n".join(lines))
       sys.exit(1)
 
     def getTerminal(self):
@@ -390,7 +418,6 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
 
     def _term_content_changed(self, term):
         " called when the *visible* part of the terminal changes "
-
         # get the current visible text, 
         current_text = self._term.get_text(lambda a,b,c,d: True)
         # see what we have currently and only print stuff that wasn't
@@ -561,12 +588,16 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
             return True
         return False
 
-    def askYesNoQuestion(self, summary, msg):
+    def askYesNoQuestion(self, summary, msg, default='No'):
         msg = "<big><b>%s</b></big>\n\n%s" % (summary,msg)
         dialog = gtk.MessageDialog(parent=self.window_main,
                                    flags=gtk.DIALOG_MODAL,
                                    type=gtk.MESSAGE_QUESTION,
                                    buttons=gtk.BUTTONS_YES_NO)
+        if default == 'No':
+          dialog.set_default_response(gtk.RESPONSE_NO)
+        else:
+          dialog.set_default_response(gtk.RESPONSE_YES)
         dialog.set_markup(msg)
         res = dialog.run()
         dialog.destroy()
@@ -584,6 +615,10 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
             return True
         return False
 
+    def processEvents(self):
+        while gtk.events_pending():
+            gtk.main_iteration()
+
     def on_window_main_delete_event(self, widget, event):
         self.dialog_cancel.set_transient_for(self.window_main)
         self.dialog_cancel.realize()
@@ -591,9 +626,7 @@ class DistUpgradeViewGtk(DistUpgradeView,SimpleGladeApp):
         res = self.dialog_cancel.run()
         self.dialog_cancel.hide()
         if res == gtk.RESPONSE_CANCEL:
-            #FIXME: this does not work correctly and leaves a stalled
-            #       dist-upgrade.py process
-            self.destroy()
+            sys.exit(1)
         return True
 
 if __name__ == "__main__":
@@ -604,8 +637,12 @@ if __name__ == "__main__":
 
   cache = apt.Cache()
   for pkg in sys.argv[1:]:
-    cache[pkg].markInstall()
+    if cache[pkg].isInstalled:
+      cache[pkg].markDelete()
+    else:
+      cache[pkg].markInstall()
   cache.commit(fp,ip)
+  gtk.main()
   sys.exit(0)
   
   #sys.exit(0)
