@@ -62,14 +62,14 @@ class KDECdromProgressAdapter(apt.progress.CdromProgress):
     """ Report the cdrom add progress """
     def __init__(self, parent):
         self.status = parent.window_main.label_status
-        self.progress = parent.window_main.progressbar_cache
+        self.progressbar = parent.window_main.progressbar_cache
         self.parent = parent
 
     def update(self, text, step):
         """ update is called regularly so that the gui can be redrawn """
         if text:
           self.status.setText(text)
-        self.progressbar.setProgress(percent)
+        self.progressbar.setProgress(step/float(self.totalSteps))
         KApplication.kApplication().processEvents()
 
     def askCdromName(self):
